#!/bin/bash

RECORD_LOG="/root/pear/log/network.log"
OS=`grep "^ID=" /etc/os-release | cut -d '=' -f 2- | sed 's|"||g'`

function set_ip_route(){
    local nic=$1
    local addr=$2
    local gate=$3
    local mask=$4
    if [ -z "$nic" ] || [ -z "$addr" ] || [ -z "$gate" ] || [ -z "$mask" ]; then
        return
    fi
    if [ -z "$(cat /etc/iproute2/rt_tables | grep "${nic}" | grep -v '#')" ];then
        echo ${mask} ${nic} >> /etc/iproute2/rt_tables # 注册路由表
        sleep 2 # 给系统响应时间
    fi
    echo `date "+%Y-%m-%d %H:%M:%S"` $nic $addr $gate $mask >> ${RECORD_LOG}

    ip route flush table $nic # 清空刷新路由表
    ip route add default dev $nic table $nic # 添加默认路由
    ip rule add from $addr table $nic pref 200 # 强制 此地址 走此 $nic路由表
    ip rule add to $addr lookup $nic pref 200 # 让访问 此地址 的走 $nic路由表
    ip route replace default dev $nic table $nic via $gate # 添加网关

}

function set_centos (){
    if [[ "$restart" == "1" ]]; then
        echo `date "+%Y-%m-%d %H:%M:%S"`" systemctl restart network" >> ${RECORD_LOG}
        systemctl restart network
        sleep 5
    fi
    
    
    local files=$(ls /etc/sysconfig/network-scripts/| grep -E ifcfg-"eth|eno|enp")
    local files=($files)
    local mask=100

    for file in "${files[@]}"; do
    
        local nic=$(cat /etc/sysconfig/network-scripts/${file} | grep DEVICE | grep -v "#" | tr -d "=" | tr -d "'" | tr -d " " | sed 's/device//i')
        local addr=$(cat /etc/sysconfig/network-scripts/${file} | grep IPADDR | grep -v "#" | tr -d "IPADDR" | tr -d "=" | tr -d "'" | tr -d " ")
        local gate=$(cat /etc/sysconfig/network-scripts/${file} | grep GATEWAY | grep -v "#" | tr -d "GATEWAY" | tr -d "=" | tr -d "'" | tr -d " ")

        if [ -n "$appointNic" ] && [[ "$nic" != "$appointNic" ]];then
            let mask+=10
            continue
        fi
        # 判断是否有ip
        if [[ "$addr" == "$(ifconfig $nic | grep inet | grep -v inet6 | awk '{print $2}')" ]] ;then
            set_ip_route $nic $addr $gate $mask
            sleep 3
        fi
        
        let mask+=10
        
    done
    
}

function set_ubuntu (){
    if [[ "$restart" == "1" ]]; then
        echo `date "+%Y-%m-%d %H:%M:%S"`" netplan apply" >> ${RECORD_LOG}
        netplan apply
        sleep 5
    fi

    local rowNum=$(cat -n ${filename} | grep -E "eth|eno|enp" | grep -v "ethernets" | awk '{print $1}')
    local rowNum=(${rowNum//\n/})
    local endRow=$(wc -l ${filename} | awk '{print $1}')
    local rowNum[${#rowNum[@]}]=$endRow
    local mask=100

    for(( i=0;i<${#rowNum[@]}-1;i++)) do #${#array[@]}获取数组长度用于循环

        local start=${rowNum[${i}]}
        local end=${rowNum[${i}+1]}

        local nic=$(sed -n "${start}p" $filename | tr -d ':' | tr -d ' ')
        local addr=$(ifconfig $nic|grep inet| grep -v inet6 | awk '{print $2}')
        local gate=$(sed -n "${start},${end}p" $filename | grep "gateway4" | tr -d " " | sed 's/gateway4://i')
        echo $nic $addr $gate $mask
        if [ -n "$appointNic" ] && [[ "$nic" != "$appointNic" ]];then
            let mask+=10
            continue
        fi
        if [[ "$addr" == "$(ifconfig $nic | grep inet | grep -v inet6 | awk '{print $2}')" ]] ;then
            set_ip_route $nic $addr $gate $mask
            sleep 3
        fi
        let mask+=10
    done

}

if [[  "$(cat /root/pear/var/network_connection_type)" != "static"  ]]; then
    exit 0;
fi


filename=/etc/netplan/01-network-manager-all.yaml
restart=0
appointNic=""

ARGS=`getopt -o :f:rn: --long file:,restart,nic: -n "$0" -- "$@"`
if [ $? != 0 ]; then
    echo "Terminating..."
    exit 1
fi

echo ARGS=[$ARGS]
#将规范化后的命令行参数分配至位置参数（$1,$2,...)
eval set -- "${ARGS}"
echo formatted parameters=[$@]

while true
do
    case "$1" in
        -f | --file)
            filename=$2
            shift 2;
        ;;
        -r | --restart)
            restart=1
            shift;
        ;;
        -n | --nic)
            appointNic=$2
            shift 2
        ;;
        --)
            shift;
            break;
        ;;
        ?)
        echo "未知参数"
        exit;
        ;;
    esac
done


echo `date "+%Y-%m-%d %H:%M:%S"` "file=$filename" 
echo `date "+%Y-%m-%d %H:%M:%S"` "restart=$restart"
echo `date "+%Y-%m-%d %H:%M:%S"` "appointNic=$appointNic"



if [ "${OS}" = "centos" ]; then
    echo `date "+%Y-%m-%d %H:%M:%S"`" set centos"
    set_centos
elif [ "${OS}" = "ubuntu" ]; then
    echo `date "+%Y-%m-%d %H:%M:%S"`" set ubuntu"
    set_ubuntu
fi