#!/bin/bash

# 因为网卡稳定性不够，会出现以下几种状态
# 1. 由于网卡松动丢失 ip rule 规则 （此问题主要依靠网卡启动时的回调脚本解决） 
#    - err: 100
# 2. 网卡出现假死状况
#   - 出现情况
#       - 强制重刷路由可解决
#            - err: 101
#       - 重启网卡可解决
#            - err: 102
# 3. 上游断线，导致ip rule失效
#   - err: 103 

mkdir -p /root/pear/log
mkdir -p /root/pear/var

netFiles="/etc/pear/pear_monitor/nic.conf"
# echo "Pear_Five_Minutes_Check"
RECORD_LOG="/root/pear/log/network.log"

function restart_net () {   
    nics=`cat ${netFiles} | tr ',' ' '`
    for nic in ${nics[@]}; do
        echo ${nic}
        # if ping -I ${nic} -c 4 -w 6 qq.com ; then
        #     echo "Ping Success, no need restart " ${nic}
        # else
        #     echo "Ping Failed, restart " ${nic}
        #     /root/pear/network/static/setroute.sh
        # fi
        detect_nic $nic
    done
}
# 返回 1：网卡请求成功！
# 返回 0： 网卡请求失败！
function request_with_nic(){
    nic=$1
    # 获取ip方式
    # https://api.myip.com
    # https://ipinfo.io
    # https://cip.cc
    # https://myip.ipip.net
    # https://ifconfig.me
    # http://members.3322.org/dyndns/getip

    # 此api可能请求到ipv6，导致不匹配
    result=$(curl --interface ${nic}  -X GET https://api.myip.com)
    ip=$(echo $result | jq .ip | tr -d '"')
    # ip="192.168.2.153"
    # 判断是否返回ip
    if [ -z "$ip" ]; then
        result=$(curl --interface ${nic}  -X GET https://ipinfo.io)
        ip=$(echo $result | jq .ip | tr -d '"')
    fi
    if [ -z "$ip" ]; then
        ip=$(curl --interface ${nic}  -X GET https://ifconfig.me)
    fi

    
    # if [ "curl -sL --interface eth1 -w "%{http_code}" "https://ifconfig.me" -o /dev/null" = "200" ]; then
    # 判断 ip 是否相等
    # echo "ifconfig $(ifconfig ${nic} | grep inet | grep -v inet6 | awk '{print $2}')"
    # echo "ip ${ip}"

    if [ -n "$ip" ]; then 
    
    # if [[ "$ip" == "$(ifconfig ${nic} | grep inet | grep -v inet6 | awk '{print $2}')" ]]; then 
        echo 1
        return 1
    fi
    echo 0
    return 0
}

function detect_nic(){
    nic=$1

    count=$(cat /root/pear/var/$nic)
    checkIPRule=$(ip ro ls tab ${nic})
    if [ -z "${checkIPRule}" ] && [[ ${checkIPRule} != "default via *" ]];then
        echo `date "+%Y-%m-%d %H:%M:%S"`" 路由掉线 ${nic} err: 100" >> ${RECORD_LOG}
        /root/pear/network/static/setroute.sh --nic ${nic}
    fi

    result=`request_with_nic $nic`
    if [ $result -eq 0 ]; then 
        echo `date "+%Y-%m-%d %H:%M:%S"`" 网卡请求失败 ${nic}" >> ${RECORD_LOG}
        count=$(($count+1))
        echo $count > /root/pear/var/$nic
    else
        echo `date "+%Y-%m-%d %H:%M:%S"`" 网卡状态正常 ${nic}" >> ${RECORD_LOG}
        echo 0 > /root/pear/var/$nic
        return; # 已恢复
    fi

    if [ $count -eq 5 ]; then 
        if [ -z "${checkIPRule}" ] && [[ ${checkIPRule} != "default via *" ]];then
            echo `date "+%Y-%m-%d %H:%M:%S"`" 强制重刷路由 ${nic}" >> ${RECORD_LOG}
            /root/pear/network/static/setroute.sh -r
        fi
        result=`request_with_nic $nic`
        if [ $result -eq 1 ]; then 
            echo `date "+%Y-%m-%d %H:%M:%S"`" 强制重启网卡后访问 ${nic} err: 101" >> ${RECORD_LOG}
        else
            echo `date "+%Y-%m-%d %H:%M:%S"`" 网络重连失败 ${nic}" >> ${RECORD_LOG}
        fi
        
        echo `date "+%Y-%m-%d %H:%M:%S"`" 尝试重启网卡 ${nic}" >> ${RECORD_LOG}
        ifconfig ${nic} down
        sleep 5
        ifconfig ${nic} up
        /root/pear/network/static/setroute.sh -r

        result=`request_with_nic $nic`
        if [ $result -eq 1 ]; then 
            echo `date "+%Y-%m-%d %H:%M:%S"`" 网卡假死！已恢复 ${nic} err: 102" >> ${RECORD_LOG}
        else
            echo `date "+%Y-%m-%d %H:%M:%S"`" 网络重连失败 ${nic} err: 103" >> ${RECORD_LOG}
        fi
        echo 0 > /root/pear/var/$nic
    fi
}

if [[  "$(cat /root/pear/var/network_connection_type)" != "static"  ]]; then
    exit 0;
fi

restart_net
